package department;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.drools.KnowledgeBase;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.logger.KnowledgeRuntimeLogger;
import org.drools.logger.KnowledgeRuntimeLoggerFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.workitem.wsht.HornetQHTWorkItemHandler;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.ContentData;
import org.jbpm.task.service.SyncTaskServiceWrapper;
import org.jbpm.task.Content;
import org.jbpm.task.Task;
import org.jbpm.task.TaskService;
import org.jbpm.task.service.hornetq.AsyncHornetQTaskClient;
import org.jbpm.task.utils.ContentMarshallerHelper;



public class DepartmentOfEductaionProcess {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			//load the knowledge base
			KnowledgeBase kBase=readKnowledgeBase();
			StatefulKnowledgeSession kSession=kBase.newStatefulKnowledgeSession();
			KnowledgeRuntimeLogger logger =KnowledgeRuntimeLoggerFactory.newThreadedFileLogger(kSession, "test", 1000);
			
			HornetQHTWorkItemHandler humanHandler=new HornetQHTWorkItemHandler(kSession);
			
			kSession.getWorkItemManager().registerWorkItemHandler("Human Task", humanHandler);
			
			//start a new process instance
			Map<String, Object> params=new HashMap<String, Object>();
			params.put("userId", "principal");
			params.put("description", "Need to evaluate the school");
			
			kSession.startProcess("departmentOfEductation.bpmn", params);
			
			SystemEventListenerFactory.setSystemEventListener(new SystemEventListener ());
			
			//we can reuse the client used by the work item hander
			
			TaskService taskClient=new SyncTaskServiceWrapper(new AsyncHornetQTaskClient("DepartmentOfEductaionProcess-testClient"));
			
			taskClient.connect("127.0.0.1", 5445);
			
			Thread.sleep(1000);
			//"DOEOfficial" reviews request
			List<String> groups= new ArrayList<String>();
			groups.add("officials");
			
			TaskSummary task1=taskClient.getTasksAssignedAsPotentialOwner("DOEOfficial", groups,"en-UK").get(0);
			System.out.println("DOEOfficial execute task" + task1.getName()+ "(" + task1.getId() + ": " + task1.getDescription() + ")");
			
			taskClient.claim(task1.getId(), "DOEOfficial",groups);
			
			Thread.sleep(2000);
			
			taskClient.start(task1.getId(), "DOEOfficial");
			
			Map<String, Object> results= new HashMap<String, Object>();
			results.put("comment", "Agreed, School evaluation will be done");
			results.put("outcome", "Accept");
			
			ContentData contentData=ContentMarshallerHelper.marshal(results, null, null);
			
			taskClient.complete(task1.getId(), "DOEOfficial", contentData);
			
			Thread.sleep(2000);
			
			//"departmentOfEducation" approves result
			
			TaskSummary task2=taskClient.getTasksAssignedAsPotentialOwner("departmentOfEducation", "en-UK").get(0);
			System.out.println("departmentOfEducation execute task" + task2.getName() + "(" +": " + task2.getDescription() + ")");
			
			taskClient.start(task2.getId(), "departmentOfEducation");
			results=new HashMap<String, Object>();
			results.put("outcome", "Agree");
			
			contentData=ContentMarshallerHelper.marshal(results, null, null);
			
			taskClient.complete(task2.getId(), "departmentOfEducation", contentData);
			
			Thread.sleep(2000);
			
			//"principal" as head of department reviews the request
			groups=new ArrayList<String>();
			groups.add("PM");
			
			TaskSummary task3=taskClient.getTasksAssignedAsPotentialOwner("principal", groups,"en-UK").get(0);
            System.out.println("principal executing task " + task3.getName() + "(" + task3.getId() + ": " + task3.getDescription() + ")");
            
            taskClient.claim(task3.getId(), "principal",groups);
            
            taskClient.start(task3.getId(), "principal");
            
            results=new HashMap<String, Object>();
            results.put("outcome", "Agree");
            
            contentData=ContentMarshallerHelper.marshal(results, null, null);
            
            taskClient.complete(task3.getId(), "principal", contentData);
            
            Thread.sleep(2000);
            
            //"DOEOfficial" gets notification
            
            TaskSummary task4=taskClient.getTasksAssignedAsPotentialOwner("DOEOfficial", "en-UK").get(0);
            
            System.out.println("DOEOfficial executing task " + task4.getName() + "(" + task4.getId() + ": " + task4.getDescription() + ")");
			
			taskClient.start(task4.getId(), "DOEOfficial");
			
			Task task=taskClient.getTask(task4.getId());
			
			Content content=taskClient.getContent(task.getTaskData().getDocumentContentId());
			
			Object result=ContentMarshallerHelper.unmarshall(null, content.getContent(), null, null);
			
			Map<?, ?>map=(Map<?, ?>)result;
			
			for(Map.Entry<?, ?> entry: map.entrySet())
			{
				System.out.println(entry.getKey() + "=" + entry.getValue());
			}
			
			taskClient.complete(task4.getId(), "DOEOfficial", null);
			
			Thread.sleep(2000);
			taskClient.disconnect();
			humanHandler.dispose();
			logger.close();
			System.exit(0);
		} catch (Throwable t) {
			t.printStackTrace();
		}

	}

	private static KnowledgeBase readKnowledgeBase()throws Exception
	{
		KnowledgeBuilder kBuilder=KnowledgeBuilderFactory.newKnowledgeBuilder();
		kBuilder.add(ResourceFactory.newClassPathResource("DOEOfficial.bpmn"), ResourceType.BPMN2);
		return kBuilder.newKnowledgeBase();
	}
    private static class SystemEventListener implements org.drools.SystemEventListener {

        public void debug(String arg0) {
        }

        public void debug(String arg0, Object arg1) {
        }

        public void exception(Throwable arg0) {
        }

        public void exception(String arg0, Throwable arg1) {
        }

        public void info(String arg0) {
        }

        public void info(String arg0, Object arg1) {
        }

        public void warning(String arg0) {
        }

        public void warning(String arg0, Object arg1) {
        }
    }

}
